@@ -5,7 +5,7 @@ use web_sys::HtmlCanvasElement;
 use web_sys::HtmlDivElement;
 use web_sys::WebGl2RenderingContext as GL;
 use web_sys::WebGlProgram;
-use yew::services::{RenderService, Task};
+use yew::services::{ConsoleService, RenderService, Task};
 use yew::{html, Component, ComponentLink, Html, NodeRef, ShouldRender};
 
 use crate::coord::*;
@@ -50,9 +50,9 @@ const SURFACE_INDICES: [[u8; 4]; 6] = [
 fn make_color_list(colors: [Vec4; 6]) -> [Vec4; 54] {
     let mut out = [Vec4::default(); 54];
     for surface in SURFACE_LIST {
-        for x in -1..=1 {
-            for y in -1..=1 {
-                for z in -1..=1 {
+        for x in 0..3 {
+            for y in 0..3 {
+                for z in 0..3 {
                     let p = Piece(x, y, z);
                     if let Some(SurfaceIndex(s, i, j)) = surface_index_of(p, surface) {
                         let b = surface_number(s, i, j);
@@ -82,6 +82,7 @@ impl RotationProgress {
     }
 }
 use std::collections::HashMap;
+use std::iter::FromIterator;
 pub struct Cube {
     canvas: Option<HtmlCanvasElement>,
     gl: Option<GL>,
@@ -146,9 +147,9 @@ impl Component for Cube {
         }
 
         let mut cache = HashMap::new();
-        for x in -1..=1 {
-            for y in -1..=1 {
-                for z in -1..=1 {
+        for x in 0..3 {
+            for y in 0..3 {
+                for z in 0..3 {
                     let piece = Piece(x, y, z);
                     cache.insert(piece, PieceData::default());
                 }
@@ -164,18 +165,17 @@ impl Component for Cube {
             canvas_node_ref: NodeRef::default(),
             fps_node_ref: NodeRef::default(),
             render_loop: None,
-            state: props.init_state,
             color_list,
 
             prev_timestamp: None,
 
-            command_queue,
-            cur_rotation: None,
-            next_state: props.init_state,
-
-            blacklist: props.blacklist,
-
             cache,
+
+            cur_rotation: None,
+            state: props.init_state,
+            next_state: props.init_state,
+            command_queue,
+            blacklist: props.blacklist,
         }
     }
 
@@ -216,11 +216,11 @@ impl Component for Cube {
             // The geometry won't change.
             let edge = 2.;
             let e = 0.5 * edge;
-            for x in -1..=1 {
-                for y in -1..=1 {
-                    for z in -1..=1 {
-                        let piece = Piece(x, y, z);
-                        let piece_center = 2. * vec3(x as f32 * e, y as f32 * e, z as f32 * e);
+            for x in 0..3 {
+                for y in 0..3 {
+                    for z in 0..3 {
+                        let piece = Piece(x as u8, y as u8, z as u8);
+                        let piece_center = 2. * vec3((x-1) as f32 * e, (y-1) as f32 * e, (z-1) as f32 * e);
                         let cube = CubePiece::new(piece_center, 0.95 * edge);
                         let mut vertex_pos_list = vec![];
                         let mut index_list = vec![];
@@ -269,7 +269,17 @@ impl Component for Cube {
         }
     }
 
-    fn change(&mut self, _props: Self::Properties) -> ShouldRender {
+    fn change(&mut self, props: Self::Properties) -> ShouldRender {
+        self.state = props.init_state;
+        self.next_state = props.init_state;
+        self.blacklist = props.blacklist;
+        self.command_queue = VecDeque::from_iter(props.command_list);
+        self.cur_rotation = None;
+
+        let render_frame = self.link.callback(Msg::Render);
+        let handle = RenderService::request_animation_frame(render_frame);
+        self.render_loop = Some(Box::new(handle));
+
         false
     }
 }
@@ -297,10 +307,12 @@ impl Cube {
             if let Some(head) = self.command_queue.pop_front() {
                 let rot = coord::rotation_of(head);
                 let mut pieces = HashSet::new();
-                for i in rot.indices {
-                    let plane = coord::RotationPlane(rot.axis, i);
-                    for x in coord::piece_group_of(plane) {
-                        pieces.insert(*x);
+                for i in 0..3 {
+                    if rot.indices & (1 << i) > 0 {
+                        let plane = coord::RotationPlane(rot.axis, i);
+                        for x in coord::piece_group_of(plane) {
+                            pieces.insert(*x);
+                        }
                     }
                 }
                 let new_rot = RotationProgress {
@@ -311,13 +323,13 @@ impl Cube {
                     start_time: timestamp,
                 };
                 self.cur_rotation = Some(new_rot);
-                self.next_state = matrix::of(head) * self.state;
+                self.next_state = matrix::of(coord::rotation_of(head)) * self.state;
             }
 
             // colors should be updated for every move.
-            for x in -1..=1 {
-                for y in -1..=1 {
-                    for z in -1..=1 {
+            for x in 0..3 {
+                for y in 0..3 {
+                    for z in 0..3 {
                         let piece = Piece(x, y, z);
                         let mut vertex_color_list = vec![];
                         for surface in coord::SURFACE_LIST {
@@ -349,7 +361,7 @@ impl Cube {
 
         // The code below will be executed for every frame.
 
-        gl.clear_color(0., 0., 0., 1.);
+        gl.clear_color(0., 0., 0., 0.2);
         gl.clear(GL::COLOR_BUFFER_BIT | GL::DEPTH_BUFFER_BIT);
         gl.enable(GL::DEPTH_TEST);
 
@@ -375,9 +387,9 @@ impl Cube {
             m_projection.as_slice(),
         );
 
-        for x in -1..=1 {
-            for y in -1..=1 {
-                for z in -1..=1 {
+        for x in 0..3 {
+            for y in 0..3 {
+                for z in 0..3 {
                     let piece = Piece(x, y, z);
 
                     let piece_data = &self.cache.get(&piece).unwrap();
